package com.ectrip.cyt.utils;

import com.ectrip.cyt.exception.BusinessException;
import org.apache.commons.codec.binary.Base64;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;
import org.bouncycastle.util.encoders.Hex;

import javax.crypto.*;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.security.*;

/**
 * @Author zhoumengjun
 * @Description sm4加密算法工具类
 * @Date 2023/7/12 10:47
 */
public class SM4Tools {

    static {
        Security.addProvider(new BouncyCastleProvider());
    }

    public static final String ALGORITHM = "SM4";

    // 加密算法/分组加密模式/分组填充方式
    // PKCS5Padding-以8个字节为一组进行分组加密
    // 定义分组加密模式使用：PKCS5Padding
    public static final String SM4_CBC_PCK_ALG = "SM4/CBC/PKCS5Padding";

    // 128-32位16进制；256-64位16进制
    public static final int DEFAULT_KEY_SIZE = 128;


    public static String encryptCBC(String content, String key,String ivBase64){
        try {
            String cipherText = "";
            // 16进制字符串-->byte[]
            byte[] keyData = ByteUtils.fromHexString(key);
            // String-->byte[]
            byte[] srcData = content.getBytes(OtaUtils.ENCODE_UTF8);
            // 加密后的数组
            Cipher cipher = generateCBCCipher(SM4_CBC_PCK_ALG, Cipher.ENCRYPT_MODE, keyData,SecurityTools.decodeIv(ivBase64));
            byte[] cipherArray = cipher.doFinal(srcData);
            // byte[]-->hexString
            cipherText = ByteUtils.toHexString(cipherArray);
            return cipherText;
        }catch (Exception e){
            throw new BusinessException("SM4加密失败");
        }
    }

    public static String decryptCBC(String content, String key,String ivBase64){
        try {
            // 用于接收解密后的字符串
            String decryptStr = "";
            // hexString-->byte[]
            byte[] keyData = ByteUtils.fromHexString(key);
            // hexString-->byte[]
            byte[] cipherData = ByteUtils.fromHexString(content);
            // 解密
            Cipher cipher = generateCBCCipher(SM4_CBC_PCK_ALG, Cipher.DECRYPT_MODE, keyData,SecurityTools.decodeIv(ivBase64));
            byte[] srcData = cipher.doFinal(cipherData);
            // byte[]-->String
            decryptStr = new String(srcData, OtaUtils.ENCODE_UTF8);
            return decryptStr;
        }catch (Exception e){
            throw new BusinessException("SM4解密失败");
        }
    }

    /**
     * 生成ECB暗号
     *
     * @param algorithmName 算法名称
     * @param mode          模式
     * @param key
     * @return
     * @throws Exception
     * @explain ECB模式（电子密码本模式：Electronic codebook）
     */
    private static Cipher generateCBCCipher(String algorithmName, int mode, byte[] key,byte[] iv) throws NoSuchPaddingException, NoSuchAlgorithmException, NoSuchProviderException, InvalidAlgorithmParameterException, InvalidKeyException {
        Cipher cipher = Cipher.getInstance(algorithmName, BouncyCastleProvider.PROVIDER_NAME);
        Key sm4Key = new SecretKeySpec(key, ALGORITHM);
        IvParameterSpec ivParameterSpec = new IvParameterSpec(iv);
        cipher.init(mode, sm4Key,ivParameterSpec);
        return cipher;
    }

    public static void main(String[] args) {
        try {
            String key = SecurityTools.generateKey(OtaUtils.SECURITY_AES_CBC);
            System.out.println(key);
            String iv = SecurityTools.generatorIv();
            System.out.println(iv);
            String data ="<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n" +
                    "<request xmlns=\"http://tour.ectrip.com/2014/QMRequestDataSchema\">\n" +
                    "    <header>\n" +
                    "        <application>tour.ectrip.com</application>\n" +
                    "        <processor>DataExchangeProcessor</processor>\n" +
                    "        <version>v1.0.0</version>\n" +
                    "        <bodyType>GetOrderByOTARequestBody</bodyType>\n" +
                    "        <createUser>wllvmama</createUser>\n" +
                    "        <createTime>2020-11-18 11:36:09</createTime>\n" +
                    "    </header>\n" +
                    "    <body xsi:type=\"GetOrderByOTARequestBody\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n" +
                    "        <partnerOrderId>91201118999003835</partnerOrderId>\n" +
                    "    </body>\n" +
                    "</request>\n" +
                    "\n";
            //加密
            String cipher = AesTools.encryptCBC(data, key,iv);
            System.out.println("加密后："+cipher);
            //解密
            String res = AesTools.decryptCBC(cipher, key,iv);
            System.out.println("解密后："+res);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 根据位数 自动生成密钥
     *
     * @return
     * @explain
     */
    public static String generateKey() throws Exception {
        KeyGenerator kg = KeyGenerator.getInstance(ALGORITHM, BouncyCastleProvider.PROVIDER_NAME);
        kg.init(DEFAULT_KEY_SIZE, new SecureRandom());
        byte[] bytes = kg.generateKey().getEncoded();
        return new String(Hex.encode(bytes));
    }
}
